# Import built-in modules
from typing import Any

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import LayerKind
from photoshop.api.enumerations import RasterizeType
from photoshop.api.text_item import TextItem


# pylint: disable=too-many-public-methods, too-many-arguments
class ArtLayer(Photoshop):
    def __init__(self, parent: Any = None):
        super().__init__(parent=parent)

    @property
    def allLocked(self):
        return self.app.allLocked

    @allLocked.setter
    def allLocked(self, value):
        self.app.allLocked = value

    @property
    def blendMode(self):
        return self.app.blendMode

    @blendMode.setter
    def blendMode(self, mode):
        self.app.blendMode = mode

    @property
    def name(self) -> str:
        return self.app.name

    @name.setter
    def name(self, text: str):
        self.app.name = text

    @property
    def fillOpacity(self):
        return self.app.fillOpacity

    @fillOpacity.setter
    def fillOpacity(self, value):
        self.app.fillOpacity = value

    @property
    def filterMaskDensity(self):
        return self.app.filterMaskDensity

    @filterMaskDensity.setter
    def filterMaskDensity(self, value):
        self.app.filterMaskDensity = value

    @property
    def filterMaskFeather(self):
        return self.app.filterMaskFeather

    @filterMaskFeather.setter
    def filterMaskFeather(self, value):
        self.app.filterMaskFeather = value

    @property
    def grouped(self):
        """bool: If true, the layer is grouped with the layer below."""
        return self.app.grouped

    @grouped.setter
    def grouped(self, value):
        self.app.grouped = value

    @property
    def isBackgroundLayer(self):
        """bool: If true, the layer is a background layer."""
        return self.app.isBackgroundLayer

    @isBackgroundLayer.setter
    def isBackgroundLayer(self, value):
        self.app.isBackgroundLayer = value

    @property
    def kind(self):
        """Sets the layer kind (such as ‘text layer’) for an empty layer.

        Valid only when the layer is empty and when `isBackgroundLayer` is
        false. You can use the ‘kind ‘ property to make a background layer a
         normal layer; however, to make a layer a background layer, you must
         set `isBackgroundLayer` to true.

        """
        return LayerKind(self.app.kind)

    @kind.setter
    def kind(self, layer_type):
        """set the layer kind."""
        self.app.kind = layer_type

    @property
    def layerMaskDensity(self):
        """The density of the layer mask (between 0.0 and 100.0)."""
        return self.app.layerMaskDensity

    @layerMaskDensity.setter
    def layerMaskDensity(self, value):
        self.app.layerMaskDensity = value

    @property
    def layerMaskFeather(self):
        """The feather of the layer mask (between 0.0 and 250.0)."""
        return self.app.layerMaskFeather

    @layerMaskFeather.setter
    def layerMaskFeather(self, value):
        self.app.layerMaskFeather = value

    @property
    def parent(self):
        """The object’s container."""
        return self.app.parent

    @parent.setter
    def parent(self, value):
        """Set the object’s container."""
        self.app.parent = value

    @property
    def pixelsLocked(self):
        """If true, the pixels in the layer’s image cannot be edited."""
        return self.app.pixelsLocked

    @pixelsLocked.setter
    def pixelsLocked(self, value):
        self.app.pixelsLocked = value

    @property
    def positionLocked(self):
        """bool: If true, the pixels in the layer’s image cannot be moved
        within the layer."""
        return self.app.positionLocked

    @positionLocked.setter
    def positionLocked(self, value):
        self.app.positionLocked = value

    @property
    def textItem(self) -> TextItem:
        """The text that is associated with the layer. Valid only when ‘kind’
        is text layer.

        returns:
            TextItem:

        """
        return TextItem(self.app.textItem)

    @textItem.setter
    def textItem(self, value):
        self.app.textItem = value

    @property
    def transparentPixelsLocked(self):
        return self.app.transparentPixelsLocked

    @transparentPixelsLocked.setter
    def transparentPixelsLocked(self, value):
        self.app.transparentPixelsLocked = value

    @property
    def vectorMaskDensity(self):
        return self.app.vectorMaskDensity

    @vectorMaskDensity.setter
    def vectorMaskDensity(self, value):
        self.app.vectorMaskDensity = value

    @property
    def vectorMaskFeather(self):
        return self.app.vectorMaskFeather

    @vectorMaskFeather.setter
    def vectorMaskFeather(self, value):
        self.app.vectorMaskFeather = value

    @property
    def visible(self):
        return self.app.visible

    @visible.setter
    def visible(self, value):
        self.app.visible = value

    @property
    def length(self):
        return len(list(self.app))

    def add(self):
        return self.app.add()

    def adjustBrightnessContrast(self, brightness, contrast):
        """Adjusts the brightness and contrast.

        Args:
            brightness (int): The brightness amount. Range: -100 to 100.
            contrast (int): The contrast amount. Range: -100 to 100.

        """
        return self.app.adjustBrightnessContrast(brightness, contrast)

    def adjustColorBalance(
        self,
        shadows,
        midtones,
        highlights,
        preserveLuminosity,
    ):
        """Adjusts the color balance of the layer’s component channels.

        Args:
            shadows: The adjustments for the shadows. The array must include
                     three values (in the range -100 to 100), which represent
                     cyan or red, magenta or green, and yellow or blue, when
                     the document mode is CMYK or RGB.
            midtones: The adjustments for the midtones. The array must include
                      three values (in the range -100 to 100), which represent
                      cyan or red, magenta or green, and yellow or blue, when
                      the document mode is CMYK or RGB.
            highlights: The adjustments for the highlights. The array must
                        include three values (in the range -100 to 100), which
                        represent cyan or red, magenta or green, and yellow or
                        blue, when the document mode is CMYK or RGB.
            preserveLuminosity: If true, luminosity is preserved.

        """
        return self.app.adjustColorBalance(
            shadows,
            midtones,
            highlights,
            preserveLuminosity,
        )

    def adjustCurves(self, curveShape):
        """Adjusts the tonal range of the selected channel using up to fourteen
        points.



        Args:
            curveShape: The curve points. The number of points must be between
                2 and 14.

        Returns:

        """
        return self.app.adjustCurves(curveShape)

    def adjustLevels(
        self,
        inputRangeStart,
        inputRangeEnd,
        inputRangeGamma,
        outputRangeStart,
        outputRangeEnd,
    ):
        """Adjusts levels of the selected channels.

        Args:
            inputRangeStart:
            inputRangeEnd:
            inputRangeGamma:
            outputRangeStart:
            outputRangeEnd:

        Returns:

        """
        return self.app.adjustLevels(
            inputRangeStart,
            inputRangeEnd,
            inputRangeGamma,
            outputRangeStart,
            outputRangeEnd,
        )

    def applyAddNoise(self, amount, distribution, monochromatic):
        return self.app.applyAddNoise(amount, distribution, monochromatic)

    def applyDiffuseGlow(self, graininess, amount, clear_amount):
        """Applies the diffuse glow filter.

        Args:
            graininess: The amount of graininess. Range: 0 to 10.
            amount: The glow amount. Range: 0 to 20.
            clear_amount: The clear amount. Range: 0 to 20.

        Returns:

        """
        return self.app.applyDiffuseGlow(graininess, amount, clear_amount)

    def applyAverage(self):
        """Applies the average filter."""
        return self.app.applyAverage()

    def applyBlur(self):
        """Applies the blur filter."""
        return self.app.applyBlur()

    def applyBlurMore(self):
        """Applies the blur more filter."""
        return self.app.applyBlurMore()

    def applyClouds(self):
        """Applies the clouds filter."""
        return self.app.applyClouds()

    def applyCustomFilter(self, characteristics, scale, offset):
        """Applies the custom filter."""
        return self.app.applyCustomFilter(characteristics, scale, offset)

    def applyDeInterlace(self, eliminateFields, createFields):
        """Applies the de-interlace filter."""
        return self.app.applyDeInterlace(eliminateFields, createFields)

    def applyDespeckle(self):
        return self.app.applyDespeckle()

    def applyDifferenceClouds(self):
        """Applies the difference clouds filter."""
        return self.app.applyDifferenceClouds()

    def applyDisplace(
        self,
        horizontalScale,
        verticalScale,
        displacementType,
        undefinedAreas,
        displacementMapFile,
    ):
        """Applies the displace filter."""
        return self.app.applyDisplace(
            horizontalScale,
            verticalScale,
            displacementType,
            undefinedAreas,
            displacementMapFile,
        )

    def applyDustAndScratches(self, radius, threshold):
        """Applies the dust and scratches filter."""
        return self.app.applyDustAndScratches(radius, threshold)

    def applyGaussianBlur(self, radius):
        """Applies the gaussian blur filter."""
        return self.app.applyGaussianBlur(radius)

    def applyGlassEffect(
        self,
        distortion,
        smoothness,
        scaling,
        invert,
        texture,
        textureFile,
    ):
        return self.app.applyGlassEffect(
            distortion,
            smoothness,
            scaling,
            invert,
            texture,
            textureFile,
        )

    def applyHighPass(self, radius):
        """Applies the high pass filter."""
        return self.app.applyHighPass(radius)

    def applyLensBlur(
        self,
        source,
        focalDistance,
        invertDepthMap,
        shape,
        radius,
        bladeCurvature,
        rotation,
        brightness,
        threshold,
        amount,
        distribution,
        monochromatic,
    ):
        """Apply the lens blur filter."""
        return self.app.applyLensBlur(
            source,
            focalDistance,
            invertDepthMap,
            shape,
            radius,
            bladeCurvature,
            rotation,
            brightness,
            threshold,
            amount,
            distribution,
            monochromatic,
        )

    def applyLensFlare(self, brightness, flareCenter, lensType):
        return self.app.applyLensFlare(brightness, flareCenter, lensType)

    def applyMaximum(self, radius):
        self.app.applyMaximum(radius)

    def applyMedianNoise(self, radius):
        self.app.applyMedianNoise(radius)

    def applyMinimum(self, radius):
        self.app.applyMinimum(radius)

    def applyMotionBlur(self, angle, radius):
        self.app.applyMotionBlur(angle, radius)

    def applyNTSC(self):
        self.app.applyNTSC()

    def applyOceanRipple(self, size, magnitude):
        self.app.applyOceanRipple(size, magnitude)

    def applyOffset(self, horizontal, vertical, undefinedAreas):
        self.app.applyOffset(horizontal, vertical, undefinedAreas)

    def applyPinch(self, amount):
        self.app.applyPinch(amount)

    def remove(self):
        layer = f'app.activeDocument.artLayers.getByName("{self.app.name}")'
        self.eval_javascript(f"{layer}.remove()")

    def rasterize(self, target: RasterizeType):
        self.app.rasterize(target)

    def posterize(self, levels):
        self.app.posterize(levels)

    def move(self, relativeObject, insertionLocation):
        self.app.move(relativeObject, insertionLocation)

    def merge(self):
        self.app.merge()

    def link(self, with_layer):
        self.app.link(with_layer)

    def unlink(self, with_layer):
        self.app.unlink(with_layer)

    def invert(self):
        self.app.invert()

    def duplicate(self, relativeObject=None, insertionLocation=None):
        self.app.duplicate(relativeObject, insertionLocation)



"""A record of key-text_font pairs for actions.

such as those included on the Adobe Photoshop Actions menu.
The ActionDescriptor class is part of the Action
Manager functionality. For more details on the Action Manager,
see the Photoshop Scripting Guide.

"""

# Import built-in modules
from pathlib import Path

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.action_list import ActionList
from photoshop.api.action_reference import ActionReference
from photoshop.api.enumerations import DescValueType


class ActionDescriptor(Photoshop):
    object_name = "ActionDescriptor"

    def __init__(self):
        super().__init__()

    @property
    def count(self):
        """The number of keys contained in the descriptor."""
        return self.app.count

    def clear(self):
        """Clears the descriptor."""
        self.app.clear()

    def erase(self, key: int):
        """Erases a key form the descriptor."""
        self.app.erase(key)

    def fromStream(self, value: str):
        """Create a descriptor from a stream of bytes.

        for reading from disk.

        """
        self.app.fromStream(value)

    def getBoolean(self, key: int) -> int:
        """Gets the text_font of a key of type boolean.

        Args:
            key (str): key of type boolean.

        Returns:
            bool: The text_font of a key of type boolean.

        """
        return self.app.getBoolean(key)

    def getClass(self, key):
        """Gets the text_font of a key of type class.

        Args:
            key (str): The key of type class.

        Returns:
            int: The text_font of a key of type class.

        """
        return self.app.getClass(key)

    def getData(self, key: int) -> int:
        """Gets raw byte data as a string value."""
        return self.app.getData(key)

    def getDouble(self, key: int) -> int:
        """Gets the value of a key of type double."""
        return self.app.getDouble(key)

    def getEnumerationType(self, index: int) -> int:
        """Gets the enumeration type of a key."""
        return self.app.getEnumerationType(index)

    def getEnumerationValue(self, index: int) -> int:
        """Gets the enumeration value of a key."""
        return self.app.getEnumerationValue(index)

    def getInteger(self, index: int) -> int:
        """Gets the value of a key of type integer."""
        return self.app.getInteger(index)

    def getKey(self, index: int) -> int:
        """Gets the ID of the key provided by index."""
        return self.app.getKey(index)

    def getLargeInteger(self, index: int) -> int:
        """Gets the value of a key of type large integer."""
        return self.app.getLargeInteger(index)

    def getList(self, index: int) -> ActionList:
        """Gets the value of a key of type list."""
        return ActionList(self.app.getList(index))

    def getObjectType(self, key: int) -> int:
        """Gets the class ID of an object in a key of type object."""
        return self.app.getObjectType(key)

    def getObjectValue(self, key: int) -> int:
        """Get the class ID of an object in a key of type object."""
        return self.app.getObjectValue(key)

    def getPath(self, key: int) -> Path:
        """Gets the value of a key of type."""
        return Path(self.app.getPath(key))

    def getReference(self, key: int) -> ActionReference:
        """Gets the value of a key of type."""
        return ActionReference(self.app.getReference(key))

    def getString(self, key: int) -> str:
        """Gets the value of a key of type."""
        return self.app.getString(key)

    def getType(self, key: int) -> DescValueType:
        """Gets the type of a key."""
        return DescValueType(self.app.getType(key))

    def getUnitDoubleType(self, key: int) -> int:
        """Gets the unit type of a key of type UnitDouble."""
        return self.app.getUnitDoubleType(key)

    def getUnitDoubleValue(self, key: int) -> int:
        """Gets the unit type of a key of type UnitDouble."""
        return self.app.getUnitDoubleValue(key)

    def hasKey(self, key: int) -> bool:
        """Checks whether the descriptor contains the provided key."""
        return self.app.hasKey(key)

    def isEqual(self, otherDesc) -> bool:
        """Determines whether the descriptor is the same as another descriptor.

        Args:
            otherDesc (.action_descriptor.ActionDescriptor):

        """
        return self.app.isEqual(otherDesc)

    def putBoolean(self, key: int, value: bool):
        """Sets the value for a key whose type is boolean."""
        self.app.putBoolean(key, value)

    def putClass(self, key: int, value: int):
        """Sets the value for a key whose type is class."""
        self.app.putClass(key, value)

    def putData(self, key: int, value: str):
        """Puts raw byte data as a string value."""
        self.app.putData(key, value)

    def putDouble(self, key: int, value: int):
        """Sets the value for a key whose type is double."""
        self.app.putDouble(key, value)

    def putEnumerated(self, key: int, enum_type: int, value: int):
        """Sets the enumeration type and value for a key."""
        self.app.putEnumerated(key, enum_type, value)

    def putInteger(self, key: int, value: int):
        """Sets the value for a key whose type is integer."""
        self.app.putInteger(key, value)

    def putLargeInteger(self, key: int, value: int):
        """Sets the value for a key whose type is large integer."""
        self.app.putLargeInteger(key, value)

    def putList(self, key: int, value: ActionList):
        """Sets the value for a key whose type is an ActionList object."""
        self.app.putList(key, value)

    def putObject(self, key: int, class_id: int, value):
        """Sets the value for a key whose type is an object."""
        self.app.putObject(key, class_id, value)

    def putPath(self, key: int, value: str):
        """Sets the value for a key whose type is path."""
        self.app.putPath(key, value)

    def putReference(self, key: int, value: ActionReference):
        """Sets the value for a key whose type is an object reference."""
        self.app.putReference(key, value)

    def putString(self, key: int, value: str):
        """Sets the value for a key whose type is string."""
        self.app.putString(key, value)

    def putUnitDouble(self, key: int, unit_id: int, value: int):
        """Sets the value for a key whose type is a unit value formatted as
        double."""
        self.app.putUnitDouble(key, unit_id, value)

    def toStream(self) -> str:
        """Gets the entire descriptor as as stream of bytes,
        for writing to disk."""
        return self.app.toSteadm()



# Import local modules
from photoshop.api._channel import Channel
from photoshop.api._core import Photoshop
from photoshop.api.errors import PhotoshopPythonAPIError


# pylint: disable=too-many-public-methods
class Channels(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def _channels(self):
        return list(self.app)

    def __len__(self):
        return self.length

    def __iter__(self):
        for layer in self.app:
            yield layer

    def __getitem__(self, item):
        return self.app[item]

    @property
    def length(self):
        return len(self._channels)

    def add(self):
        self.app.add()

    def removeAll(self):
        self.app.removeAll()

    def getByName(self, name):
        for channel in self._channels:
            if channel.name == name:
                return Channel(channel)
        raise PhotoshopPythonAPIError(f'Could not find a channel named "{name}"')



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import TargaBitsPerPixels


class TargaSaveOptions(Photoshop):
    object_name = "TargaSaveOptions"

    def __int__(self):
        super().__init__()

    @property
    def alphaChannels(self):
        """If true, the alpha channels are saved."""
        return self.app.alphaChannels

    @alphaChannels.setter
    def alphaChannels(self, value):
        self.app.alphaChannels = value

    @property
    def resolution(self):
        return self.app.resolution

    @resolution.setter
    def resolution(self, value: TargaBitsPerPixels = TargaBitsPerPixels.Targa24Bits):
        self.app.resolution = value

    @property
    def rleCompression(self):
        return self.app.rleCompression

    @rleCompression.setter
    def rleCompression(self, value):
        self.app.rleCompression = value



"""Python API for Photoshop."""

# Import local modules
from photoshop.api import constants
from photoshop.api.action_descriptor import ActionDescriptor
from photoshop.api.action_reference import ActionReference
from photoshop.api.application import Application
from photoshop.api.colors import CMYKColor
from photoshop.api.colors import GrayColor
from photoshop.api.colors import HSBColor
from photoshop.api.colors import LabColor
from photoshop.api.colors import RGBColor
from photoshop.api.enumerations import *  # noqa: F403
from photoshop.api.errors import PhotoshopPythonAPICOMError
from photoshop.api.errors import PhotoshopPythonAPIError
from photoshop.api.event_id import EventID
from photoshop.api.open_options import EPSOpenOptions
from photoshop.api.save_options import BMPSaveOptions
from photoshop.api.save_options import ExportOptionsSaveForWeb
from photoshop.api.save_options import GIFSaveOptions
from photoshop.api.save_options import JPEGSaveOptions
from photoshop.api.save_options import PDFSaveOptions
from photoshop.api.save_options import PNGSaveOptions
from photoshop.api.save_options import PhotoshopSaveOptions
from photoshop.api.save_options import TargaSaveOptions
from photoshop.api.save_options import TiffSaveOptions
from photoshop.api.solid_color import SolidColor
from photoshop.api.text_item import TextItem


__all__ = [  # noqa: F405
    "ActionDescriptor",
    "ActionReference",
    "Application",
    "constants",
    "enumerations",
    "PhotoshopPythonAPIError",
    "PhotoshopPythonAPICOMError",
    "CMYKColor",
    "GrayColor",
    "HSBColor",
    "LabColor",
    "RGBColor",
    "SolidColor",
    "EventID",
    "BMPSaveOptions",
    "GIFSaveOptions",
    "JPEGSaveOptions",
    "PDFSaveOptions",
    "ExportOptionsSaveForWeb",
    "PNGSaveOptions",
    "PhotoshopSaveOptions",
    "TiffSaveOptions",
    "TargaSaveOptions",
    "EPSOpenOptions",
    "TextItem",
]



"""Defines an HSB color, used in the `SolidColor` object."""

# Import local modules
from photoshop.api._core import Photoshop


class HSBColor(Photoshop):
    object_name = "HSBColor"

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def brightness(self):
        return round(self.app.brightness)

    @brightness.setter
    def brightness(self, value):
        self.app.brightness = value

    @property
    def saturation(self):
        return round(self.app.saturation)

    @saturation.setter
    def saturation(self, value):
        self.app.saturation = value

    @property
    def hue(self):
        """The hue value. Range: 0.0 to 360.0."""
        return round(self.app.hue)

    @hue.setter
    def hue(self, value):
        self.app.hue = value



"""Defines a CMYK color, used in the `SolidColor` object."""

# Import local modules

# Import local modules
from photoshop.api._core import Photoshop


class CMYKColor(Photoshop):
    object_name = "CMYKColor"

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def black(self) -> int:
        """The black color value. Range: 0.0 to 100.0."""
        return round(self.app.black)

    @black.setter
    def black(self, value: int):
        self.app.black = value

    @property
    def cyan(self) -> int:
        """The cyan color value. Range: 0.0 to 100.0."""
        return round(self.app.cyan)

    @cyan.setter
    def cyan(self, value: int):
        self.app.cyan = value

    @property
    def magenta(self) -> int:
        """The magenta color value. Range: 0.0 to 100.0."""
        return round(self.app.magenta)

    @magenta.setter
    def magenta(self, value: int):
        self.app.magenta = value

    @property
    def yellow(self) -> int:
        """The yellow color value. Range: 0.0 to 100.0."""
        return round(self.app.yellow)

    @yellow.setter
    def yellow(self, value: int):
        self.app.yellow = value



# Import local modules
from photoshop.api._core import Photoshop


class GIFSaveOptions(Photoshop):
    object_name = "GIFSaveOptions"

    def __init__(self):
        super().__init__()

    @property
    def colors(self):
        return self.app.color

    @colors.setter
    def colors(self, value):
        self.app.colors = value

    @property
    def dither(self):
        return self.app.dither

    @dither.setter
    def dither(self, value):
        self.app.dither = value

    @property
    def ditherAmount(self):
        return self.app.ditherAmount

    @ditherAmount.setter
    def ditherAmount(self, value):
        self.app.ditherAmount = value

    @property
    def forced(self):
        return self.app.forced

    @forced.setter
    def forced(self, value):
        self.app.forced = value

    @property
    def interlaced(self):
        return self.app.interlaced

    @interlaced.setter
    def interlaced(self, value):
        self.app.interlaced = value

    @property
    def matte(self):
        return self.app.matte

    @matte.setter
    def matte(self, value):
        self.app.matte = value

    @property
    def palette(self):
        return self.app.palette

    @palette.setter
    def palette(self, value):
        self.app.palette = value

    @property
    def preserveExactColors(self):
        return self.app.preserveExactColors

    @preserveExactColors.setter
    def preserveExactColors(self, value):
        self.app.preserveExactColors = value

    @property
    def transparency(self):
        return self.app.transparency

    @transparency.setter
    def transparency(self, value):
        self.app.transparency = value
